from typing import Any, ClassVar, Dict, Type

from typing_extensions import override


class MetaSingleton(type):
    """Simple implementation of a singleton using a metaclass."""

    instances: ClassVar[Dict[Type[Any], Any]] = {}

    @override
    def __call__(cls, *args: Any, **kwargs: Any) -> Any:
        if cls not in cls.instances:  # pyright: ignore[reportUnnecessaryContains]
            cls.instances[cls] = super(MetaSingleton, cls).__call__(*args, **kwargs)
        return cls.instances[cls]



from typing import Optional

from typing_extensions import Self

from kajson.class_registry import ClassRegistry
from kajson.class_registry_abstract import ClassRegistryAbstract
from kajson.singleton import MetaSingleton

KAJSON_LOGGER_CHANNEL_NAME = "kajson"


class KajsonManager(metaclass=MetaSingleton):
    """A singleton class for managing kajson operations."""

    def __init__(
        self,
        logger_channel_name: Optional[str] = None,
        class_registry: Optional[ClassRegistryAbstract] = None,
    ) -> None:
        self.logger_channel_name = logger_channel_name or KAJSON_LOGGER_CHANNEL_NAME
        self._class_registry = class_registry or ClassRegistry()

    @classmethod
    def get_instance(cls) -> Self:
        """Get the singleton instance. This will create one if it doesn't exist."""
        return cls()

    @classmethod
    def teardown(cls) -> None:
        """Destroy the singleton instance."""
        if cls in MetaSingleton.instances:
            del MetaSingleton.instances[cls]

    @classmethod
    def get_class_registry(cls) -> ClassRegistryAbstract:
        return cls.get_instance()._class_registry



# SPDX-FileCopyrightText: © 2025 Evotis S.A.S.
# SPDX-License-Identifier: Apache-2.0

from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, Type

from pydantic import BaseModel


class ClassRegistryAbstract(ABC):
    @abstractmethod
    def setup(self) -> None:
        pass

    @abstractmethod
    def teardown(self) -> None:
        pass

    @abstractmethod
    def register_class(
        self,
        class_type: Type[Any],
        name: Optional[str] = None,
        should_warn_if_already_registered: bool = True,
    ) -> None:
        pass

    @abstractmethod
    def unregister_class(self, class_type: Type[Any]) -> None:
        pass

    @abstractmethod
    def unregister_class_by_name(self, name: str) -> None:
        pass

    @abstractmethod
    def register_classes_dict(self, classes: Dict[str, Type[Any]]) -> None:
        pass

    @abstractmethod
    def register_classes(self, classes: List[Type[Any]]) -> None:
        pass

    @abstractmethod
    def get_class(self, name: str) -> Optional[Type[Any]]:
        pass

    @abstractmethod
    def get_required_class(self, name: str) -> Type[Any]:
        pass

    @abstractmethod
    def get_required_subclass(self, name: str, base_class: Type[Any]) -> Type[Any]:
        pass

    @abstractmethod
    def get_required_base_model(self, name: str) -> Type[BaseModel]:
        pass

    @abstractmethod
    def has_class(self, name: str) -> bool:
        pass

    @abstractmethod
    def has_subclass(self, name: str, base_class: Type[Any]) -> bool:
        pass
